@@ -20,63 +20,68 @@
  */
 
 #include <curses.h>
-#include <time.h>
 #include <glib.h>
+#include <time.h>
 
 #include "commons.h"
 
-#define COL_WHITE    0
+#define BLACK_CYAN   9
+#define BLACK_GREEN  8
+#define BLUE_GREEN   7
+#define COL_BLACK    4
 #define COL_BLUE     1
+#define COL_CYAN     5
 #define COL_GREEN    11
 #define COL_RED      3
-#define COL_BLACK    4
-#define COL_CYAN     5
+#define COL_WHITE    0
 #define COL_YELLOW   6
-#define BLUE_GREEN   7
-#define BLACK_GREEN  8
-#define BLACK_CYAN   9
 #define WHITE_RED    10
 
 /* Definitions checked against declarations */
-GHashTable *ht_unique_visitors = NULL;
-GHashTable *ht_requests = NULL;
-GHashTable *ht_referrers = NULL;
-GHashTable *ht_unique_vis = NULL;
-GHashTable *ht_requests_static = NULL;
+GHashTable *ht_browsers = NULL;
+GHashTable *ht_date_bw = NULL;
+GHashTable *ht_file_bw = NULL;
+GHashTable *ht_host_bw = NULL;
+GHashTable *ht_hosts_agents = NULL;
+GHashTable *ht_hosts = NULL;
+GHashTable *ht_keyphrases = NULL;
+GHashTable *ht_monthly = NULL;
 GHashTable *ht_not_found_requests = NULL;
 GHashTable *ht_os = NULL;
-GHashTable *ht_browsers = NULL;
-GHashTable *ht_hosts = NULL;
-GHashTable *ht_status_code = NULL;
+GHashTable *ht_referrers = NULL;
 GHashTable *ht_referring_sites = NULL;
-GHashTable *ht_keyphrases = NULL;
-GHashTable *ht_file_bw = NULL;
+GHashTable *ht_requests = NULL;
+GHashTable *ht_requests_static = NULL;
+GHashTable *ht_status_code = NULL;
+GHashTable *ht_unique_visitors = NULL;
+GHashTable *ht_unique_vis = NULL;
 
 enum MODULES modules;
 enum SCHEMES schemes;
 
 /* enable flags */
-int ignore_flag = 0;
-int stripped_flag = 0;
+char *ignore_host;
 int bandwidth_flag = 0;
-long long req_size = 0;
+int host_agents_list_flag = 0;
 int http_status_code_flag = 0;
-int color_scheme = 1;
+
+/* hash iteration */
+int iter_ctr = 0;
+int iter_module = 0;
 
 /* string processing */
-char *req;
-char *status_code;
-char *ignore_host;
+long long req_size = 0;
 
-/* Processing time */
-time_t now;
+/* processing time */
 time_t end_proc;
+time_t now;
 time_t start_proc;
 
-/* resizing */
+/* resizing/scheme */
+int color_scheme = 1;
+size_t real_size_y = 0;
 size_t term_h = 0;
 size_t term_w = 0;
-size_t real_size_y = 0;
 
 /* file */
 char *ifile = NULL;
@@ -132,10 +137,10 @@ char *browsers[][2] = {
    {"Firefox", "Firefox"}, {"Chrome", "Chrome"}, {"Safari", "Safari"},
    {"Opera Mini", "Opera"}, {"Opera", "Opera"}, {"Netscape", "Others"},
    {"Konqueror", "Others"}, {"Wget", "Others"}, {"w3m", "Others"},
-   {"Links", "Others"}, {"Lynx", "Others"},
+   {"ELinks", "Others"}, {"Links", "Others"}, {"Lynx", "Others"},
    {"Camino", "Others"}, {"Dillo", "Others"}, {"Kazehakase", "Others"},
    {"Iceape", "Others"}, {"K-Meleon", "Others"}, {"Galeon", "Others"},
-   {"BrowserX", "Others"}, {"ELinks", "Others"}, {"IBrowse", "Others"},
+   {"BrowserX", "Others"}, {"IBrowse", "Others"},
    {"Mosaic", "Others"}, {"midori", "Others"}, {"Midori", "Others"},
    {"Firebird", "Others"},
    /* CRAWLERS & VALIDATORS */
@@ -153,7 +158,7 @@ char *browsers[][2] = {
    {"archive.org_bot", "Crawlers"}, {"Google-Sitemaps", "Crawlers"},
    {"PostRank", "Crawlers"}, {"KaloogaBot", "Crawlers"},
    {"Twitter", "Crawlers"}, {"yacy", "Crawlers"}, {"Nutch", "Crawlers"},
-   {"ichiro", "Crawlers"}, {"Sogou", "Crawlers"},
+   {"ichiro", "Crawlers"}, {"Sogou", "Crawlers"}, {"wikiwix", "Crawlers"},
    {"KaloogaBot", "Crawlers"}, {"Mozilla", "Others"}
 };
 
@@ -169,8 +174,8 @@ char *codes[][2] = {
    {"206", "Partial Content - The partial GET has been successful"},
    {"300", "Multiple Choices - Multiple options for the resource"},
    {"301", "Moved Permanently - Resource has permanently moved"},
-   {"302", "Moved Temporarily"},
-   {"303", "See Other - The response is at a different URI"},
+   {"302", "Moved Temporarily (redirect)"},
+   {"303", "See Other Document - The response is at a different URI"},
    {"304", "Not Modified - Resource has not been modified"},
    {"305", "Use Proxy - Can only be accessed through the proxy"},
    {"307", "Temporary Redirect - Resource temporarily moved"},
@@ -187,8 +192,8 @@ char *codes[][2] = {
    {"410", "Gone - Resource requested is no longer available"},
    {"411", "Length Required - Invalid Content-Length"},
    {"412", "Precondition Failed - Server does not meet preconditions"},
-   {"413", "Request entity Too Long"},
-   {"414", "Request-URI Too Long"},
+   {"413", "Requested Entity Too Long"},
+   {"414", "Requested Filename Too Long"},
    {"415", "Unsupported Media Type - Media type is not supported"},
    {"416", "Requested Range Not Satisfiable - Cannot supply that portion"},
    {"417", "Expectation Failed"},
