@@ -23,18 +23,18 @@
 #define _LARGEFILE64_SOURCE
 #define _FILE_OFFSET_BITS 64
 
-#include <string.h>
-#include <stdlib.h>
+#include <assert.h>
+#include <ctype.h>
 #include <curses.h>
 #include <glib.h>
-#include <stdint.h>
-#include <sys/types.h>
-#include <menu.h>
-#include <sys/stat.h>
-#include <assert.h>
-#include <unistd.h>
-#include <ctype.h>
 #include <locale.h>
+#include <menu.h>
+#include <stdint.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #include "error.h"
 #include "alloc.h"
@@ -42,6 +42,7 @@
 #include "util.h"
 #include "ui.h"
 #include "parser.h"
+#include "settings.h"
 
 static WINDOW *header_win, *main_win, *my_menu_win, *help_win, *schemes_win;
 
@@ -67,11 +68,11 @@ init_struct (void)
    if ((logger = malloc (sizeof (*logger))) == NULL)
       return NULL;
 
-   logger->total_process = 0;
-   logger->total_invalid = 0;
-   logger->counter = 0;
    logger->alloc_counter = 0;
+   logger->counter = 0;
    logger->current_module = 1;
+   logger->total_invalid = 0;
+   logger->total_process = 0;
 
    return logger;
 }
@@ -81,16 +82,20 @@ cmd_help (void)
 {
    printf ("\nGoAccess - %s\n\n", GO_VERSION);
    printf ("Usage: ");
-   printf ("goaccess [ -b ][ -s ][ -e IP_ADDRESS][ -f log_file ]\n\n");
+   printf ("goaccess [ -b ][ -s ][ -e IP_ADDRESS][ - a ]< -f log_file >\n\n");
    printf ("The following options can also be supplied to the command:\n\n");
-   printf ("  -f  - Path to input log <filename> \n");
-   printf ("  -b  - Enable total bandwidth consumption. To achieve\n");
-   printf ("        faster parsing, do not enable this flag. \n");
-   printf ("  -s  - Enable/report HTTP status codes. To avoid overhead\n");
-   printf ("        while parsing, this has been disabled by default. \n");
-   printf ("  -e  - Exclude an IP from being counted under the HOST\n");
-   printf ("        module. This has been disabled by default. \n\n");
-   printf ("For more details visit: http://goaccess.prosoftcorp.com \n\n");
+   printf (" -f <argument> - Path to input log file.\n");
+   printf (" -b            - Enable total bandwidth consumption.\n");
+   printf ("                 For faster parsing, don't enable this flag.\n");
+   printf (" -s            - Enable HTTP status codes report.\n");
+   printf ("                 For faster parsing, don't enable this flag.\n");
+   printf (" -a            - Enable a List of User-Agents by host.\n");
+   printf ("                 For faster parsing, don't enable this flag.\n");
+   printf (" -e <argument> - Exclude an IP from being counted under the\n");
+   printf ("                 HOST module. Disabled by default.\n\n");
+   printf ("For more details visit: http://goaccess.prosoftcorp.com\n");
+   printf ("GoAccess Copyright (C) 2009-2010 GNU GPL'd, by Gerardo Orellana");
+   printf ("\n\n");
    exit (EXIT_FAILURE);
 }
 
@@ -108,19 +113,22 @@ house_keeping (struct logger *logger, struct struct_display **s_display)
    free (s_display);
    free (logger);
 
-   g_hash_table_destroy (ht_unique_vis);
+   g_hash_table_destroy (ht_browsers);
+   g_hash_table_destroy (ht_date_bw);
+   g_hash_table_destroy (ht_file_bw);
+   g_hash_table_destroy (ht_host_bw);
+   g_hash_table_destroy (ht_hosts);
+   g_hash_table_destroy (ht_keyphrases);
+   g_hash_table_destroy (ht_monthly);
+   g_hash_table_destroy (ht_not_found_requests);
+   g_hash_table_destroy (ht_os);
    g_hash_table_destroy (ht_referrers);
+   g_hash_table_destroy (ht_referring_sites);
    g_hash_table_destroy (ht_requests);
    g_hash_table_destroy (ht_requests_static);
-   g_hash_table_destroy (ht_not_found_requests);
-   g_hash_table_destroy (ht_unique_visitors);
-   g_hash_table_destroy (ht_os);
-   g_hash_table_destroy (ht_browsers);
-   g_hash_table_destroy (ht_hosts);
    g_hash_table_destroy (ht_status_code);
-   g_hash_table_destroy (ht_referring_sites);
-   g_hash_table_destroy (ht_keyphrases);
-   g_hash_table_destroy (ht_file_bw);
+   g_hash_table_destroy (ht_unique_vis);
+   g_hash_table_destroy (ht_unique_visitors);
 }
 
 void
@@ -171,6 +179,7 @@ allocate_structs (int free_me)
 
    if (!http_status_code_flag)
       goto nohttpstatuscode;
+
    MALLOC_STRUCT (s_holder, g_hash_table_size (ht_status_code));
    generate_struct_data (ht_status_code, s_holder, s_display, logger, 9);
 
@@ -186,7 +195,7 @@ allocate_structs (int free_me)
 void
 render_screens (void)
 {
-   int row, col, x, y, chg = 0;
+   int row, col, chg = 0;
 
    werase (main_win);
    getmaxyx (stdscr, row, col);
@@ -216,17 +225,22 @@ render_screens (void)
               logger->current_module);
    wattroff (header_win, COLOR_PAIR (BLUE_GREEN));
    wrefresh (header_win);
+   /* no valid entries to process from the log */
+   if ((logger->total_process == 0)
+       || (logger->total_process == logger->total_invalid))
+      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                     "Nothing valid to process.");
 }
 
 static void
 get_keys (void)
 {
-   int y, x, c, quit = 0;
+   int y, x, c;
    getmaxyx (main_win, y, x);
 
-   unsigned long long size1, size2;
    char buf[BUFFER];
    FILE *fp;
+   unsigned long long size1, size2;
 
    size1 = file_size (ifile);
    while (((c = wgetch (stdscr)) != 'q')) {
@@ -360,7 +374,8 @@ get_keys (void)
           /* file has changed */
           if (size2 != size1) {
              if (!(fp = fopen (ifile, "r")))
-                error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                error_handler (__PRETTY_FUNCTION__,
+                               __FILE__, __LINE__,
                                "Unable to read log file.");
              if (!fseeko (fp, size1, SEEK_SET))
                 while (fgets (buf, BUFFER, fp) != NULL)
@@ -391,13 +406,12 @@ main (int argc, char *argv[])
       cmd_help ();
 
    opterr = 0;
-   while ((o = getopt (argc, argv, "f:e:sb")) != -1) {
+   while ((o = getopt (argc, argv, "f:e:sba")) != -1) {
       switch (o) {
        case 'f':
           ifile = optarg;
           break;
        case 'e':
-          ignore_flag = 1;
           ignore_host = optarg;
           break;
        case 's':
@@ -406,6 +420,9 @@ main (int argc, char *argv[])
        case 'b':
           bandwidth_flag = 1;
           break;
+       case 'a':
+          host_agents_list_flag = 1;
+          break;
        case '?':
           if (optopt == 'f' || optopt == 'e')
              fprintf (stderr, "Option -%c requires an argument.\n", optopt);
@@ -463,6 +480,18 @@ main (int argc, char *argv[])
    ht_file_bw =
       g_hash_table_new_full (g_str_hash, g_str_equal,
                              (GDestroyNotify) g_free, g_free);
+   ht_host_bw =
+      g_hash_table_new_full (g_str_hash, g_str_equal,
+                             (GDestroyNotify) g_free, g_free);
+   ht_date_bw =
+      g_hash_table_new_full (g_str_hash, g_str_equal,
+                             (GDestroyNotify) g_free, g_free);
+   ht_monthly =
+      g_hash_table_new_full (g_str_hash, g_str_equal,
+                             (GDestroyNotify) g_free, g_free);
+   ht_hosts_agents =
+      g_hash_table_new_full (g_str_hash, g_str_equal,
+                             (GDestroyNotify) g_free, g_free);
 
     /** 'should' work on UTF-8 terminals as long as the 
      ** user did set it to *._UTF-8 locale **/
@@ -473,12 +502,7 @@ main (int argc, char *argv[])
    if (has_colors () == FALSE)
       error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                      "Your terminal does not support color");
-   noecho ();
-   halfdelay (10);
-   nonl ();
-   intrflush (stdscr, FALSE);
-   keypad (stdscr, TRUE);
-   curs_set (0);
+   set_input_opts ();
    start_color ();
    parse_conf_file ();
    init_colors ();
