@@ -27,14 +27,15 @@
 
 #define _XOPEN_SOURCE 700
 
-#include <stdlib.h>
+#include <ctype.h>
 #include <curses.h>
-#include <string.h>
-#include <stdio.h>
-#include <time.h>
 #include <errno.h>
-#include <sys/types.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <time.h>
 
 #include "error.h"
 #include "commons.h"
@@ -106,40 +107,60 @@ verify_os_type (char *str)
 }
 
 static int
-process_request_bw (GHashTable * ht, const char *req, long long resp_size)
+process_request_bw (char *host, char *date, char *req, long long resp_size)
 {
+   GHashTable *ht = NULL;
    gpointer value_ptr;
+   int i;                       /* hash_tables to process */
 
-   if ((ht == NULL) || (req == NULL))
+   if ((host == NULL) || (date == NULL) || (req == NULL))
       return (EINVAL);
 
-   long long *ptr_value;
+   char *key = NULL;
    long long add_value;
+   long long *ptr_value;
 
-   value_ptr = g_hash_table_lookup (ht, req);
-   if (value_ptr != NULL) {
-      ptr_value = (long long *) value_ptr;
-      add_value = *ptr_value + resp_size;
-   } else
-      add_value = 0 + resp_size;
+   for (i = 0; i < BW_HASHTABLES; i++) {
+      switch (i) {
+       case 0:                 /* HOSTS bandwith */
+          ht = ht_host_bw;
+          key = host;
+          break;
+       case 1:                 /* FILES bandwith */
+          ht = ht_file_bw;
+          key = req;
+          break;
+       case 2:                 /* DATE bandwith */
+          ht = ht_date_bw;
+          key = date;
+          break;
+      }
+      value_ptr = g_hash_table_lookup (ht, key);
+      if (value_ptr != NULL) {
+         ptr_value = (long long *) value_ptr;
+         add_value = *ptr_value + resp_size;
+      } else
+         add_value = 0 + resp_size;
 
-   if (resp_size == -1)
-      add_value = 0;
+      if (resp_size == -1)
+         add_value = 0;
 
-   ptr_value = g_malloc (sizeof (long long));
-   *ptr_value = add_value;
+      ptr_value = g_malloc (sizeof (long long));
+      *ptr_value = add_value;
+
+      g_hash_table_replace (ht, g_strdup (key), ptr_value);
+   }
 
-   g_hash_table_replace (ht, g_strdup (req), ptr_value);
    return 0;
 }
 
 static int
 process_generic_data (GHashTable * ht, const char *key)
 {
-   int first = 0;
-   char *date = NULL, *ref = NULL;
-   gpointer value_ptr;
+   char *date = NULL;
    gint value_int;
+   gpointer value_ptr;
+   int first = 0;
 
    if ((ht == NULL) || (key == NULL))
       return (EINVAL);
@@ -173,10 +194,10 @@ process_generic_data (GHashTable * ht, const char *key)
       free (o_key);
       free (b_key);
       if ((date = strchr (key, '|')) != NULL) {
-         char *tmp;
-         tmp = clean_date (date);
-         process_generic_data (ht_unique_vis, tmp);
-         free (tmp);
+         char *tmp_date;
+         tmp_date = clean_date (date);
+         process_generic_data (ht_unique_vis, tmp_date);
+         free (tmp_date);
       }
    }
    return (0);
@@ -244,6 +265,41 @@ process_keyphrases (char *ref)
    return 0;
 }
 
+static int
+process_host_agents (char *host, char *agent)
+{
+   GHashTable *ht = ht_hosts_agents;
+   gpointer value_ptr;
+
+   if ((ht == NULL) || (host == NULL) || (agent == NULL))
+      return (EINVAL);
+
+   char *ptr_value, *tmp;
+
+   value_ptr = g_hash_table_lookup (ht, host);
+   if (value_ptr != NULL) {
+      ptr_value = (char *) value_ptr;
+      if (strstr (ptr_value, agent))
+         return 0;
+
+      size_t len1 = strlen (ptr_value);
+      size_t len2 = strlen (agent);
+
+      tmp = malloc (len1 + len2 + 2);
+      if (tmp == NULL)
+         error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                        "Unable to allocate memory.");
+      memcpy (tmp, ptr_value, len1);
+      tmp[len1] = '|';
+      /* NUL-terminated */
+      memcpy (tmp + len1 + 1, agent, len2 + 1);
+   } else
+      tmp = alloc_string (agent);
+
+   g_hash_table_replace (ht, g_strdup (host), tmp);
+   return 0;
+}
+
 static void
 process_unique_data (char *host, char *date, char *agent, char *status,
                      char *referrer)
@@ -262,7 +318,7 @@ process_unique_data (char *host, char *date, char *agent, char *status,
 
    if (strptime (date, "%d/%b/%Y", &tm) == NULL)
       return;
-   strftime (buf, sizeof (buf) - 1, "%Y%m%d ", &tm);
+   strftime (buf, sizeof (buf) - 1, "%Y%m%d", &tm);
 
    snprintf (unique_visitors_key, sizeof (unique_visitors_key), "%s|%s|%s",
              host, buf, agent);
@@ -278,11 +334,15 @@ process_unique_data (char *host, char *date, char *agent, char *status,
       process_generic_data (ht_status_code, status);
    }
 
-   if (ignore_flag && strcmp (host, ignore_host) == 0) {
+   if (ignore_host != NULL && strcmp (host, ignore_host) == 0) {
       /* ignore */
    } else {
       process_generic_data (ht_hosts, host);
    }
+   if (agent[strlen (agent) - 1] == '\n')
+      agent[strlen (agent) - 1] = 0;
+   if (host_agents_list_flag)
+      process_host_agents (host, agent);
    process_generic_data (ht_unique_visitors, unique_visitors_key);
 }
 
@@ -294,13 +354,13 @@ verify_static_content (char *url)
    if (strlen (url) < 5)
       return 0;
    if (!memcmp (nul - 4, ".jpg", 4) || !memcmp (nul - 4, ".JPG", 4) ||
-       !memcmp (nul - 4, ".gif", 4) || !memcmp (nul - 4, ".GIF", 4) ||
        !memcmp (nul - 4, ".png", 4) || !memcmp (nul - 4, ".PNG", 4) ||
-       !memcmp (nul - 4, ".ico", 4) || !memcmp (nul - 4, ".ICO", 4) ||
-       !memcmp (nul - 5, ".jpeg", 5) || !memcmp (nul - 4, ".JPEG", 5) ||
-       !memcmp (nul - 4, ".swf", 4) || !memcmp (nul - 4, ".SWF", 4) ||
+       !memcmp (nul - 3, ".js", 3) || !memcmp (nul - 3, ".JS", 3) ||
+       !memcmp (nul - 4, ".gif", 4) || !memcmp (nul - 4, ".GIF", 4) ||
        !memcmp (nul - 4, ".css", 4) || !memcmp (nul - 4, ".CSS", 4) ||
-       !memcmp (nul - 3, ".js", 3) || !memcmp (nul - 3, ".JS", 3))
+       !memcmp (nul - 4, ".ico", 4) || !memcmp (nul - 4, ".ICO", 4) ||
+       !memcmp (nul - 4, ".swf", 4) || !memcmp (nul - 4, ".SWF", 4) ||
+       !memcmp (nul - 5, ".jpeg", 5) || !memcmp (nul - 4, ".JPEG", 5))
       return 1;
    return 0;
 }
@@ -372,11 +432,13 @@ parse_req_size (char *line, int format)
 }
 
 static int
-parse_request (struct logger *logger, char *line, char *cpy_line)
+parse_request (struct logger *logger, char *line, char *cpy_line,
+               char **status_code, char **req)
 {
    char *ptr, *prb = NULL, *fqm = NULL, *sqm =
-      NULL, *host, *date, *ref, *hour = NULL, *h, *p;
+      NULL, *host, *date, *ref, *h, *p;
    int format = 0;
+   long long band_size = 0;
 
    host = line;
    if ((h = strchr (line, ' ')) == NULL)
@@ -420,15 +482,10 @@ parse_request (struct logger *logger, char *line, char *cpy_line)
    else
       ref = "-";
 
-   if ((hour = strchr (line, ':')) != NULL)
-      hour++;
-   else
-      hour = "-";
-
    if (!bandwidth_flag)
       goto nobanwidth;
    /* bandwidth */
-   long long band_size = parse_req_size (cpy_line, format);
+   band_size = parse_req_size (cpy_line, format);
    if (band_size != -1)
       req_size = req_size + band_size;
    else
@@ -444,16 +501,13 @@ parse_request (struct logger *logger, char *line, char *cpy_line)
    if ((ptr = strchr (date, ':')) == NULL)
       return 1;
    *ptr = '\0';
-   if ((ptr = strchr (hour, ':')) == NULL)
-      return 1;
-   *ptr = '\0';
    if ((ptr = strchr (ref, '"')) == NULL)
       ref = "-";
    else
       *ptr = '\0';
 
    /* req */
-   req = parse_req (cpy_line);
+   *req = parse_req (cpy_line);
 
    if (!http_status_code_flag)
       goto nohttpstatuscode;
@@ -461,23 +515,22 @@ parse_request (struct logger *logger, char *line, char *cpy_line)
    char *lookfor = NULL, *s_l;
    if ((lookfor = "/1.0\" ", s_l = strstr (cpy_line, lookfor)) != NULL ||
        (lookfor = "/1.1\" ", s_l = strstr (cpy_line, lookfor)) != NULL)
-      status_code = clean_status (s_l + 6);
+      *status_code = clean_status (s_l + 6);
    else
       /* perhaps something wrong with the log,
          more likely malformed request syntax */
-      status_code = alloc_string ("---");
+      *status_code = alloc_string ("---");
  nohttpstatuscode:;
 
    logger->host = host;
    logger->agent = fqm;
    logger->date = date;
-   logger->hour = hour;
    logger->referrer = ref;
-   logger->request = req;
+   logger->request = *req;
    logger->resp_size = band_size;
 
    if (http_status_code_flag)
-      logger->status = status_code;
+      logger->status = *status_code;
 
    return 0;
 }
@@ -486,12 +539,13 @@ static int
 process_log (struct logger *logger, char *line)
 {
    char *cpy_line = strdup (line);
+   char *status_code = NULL, *req;
    struct logger log;
    logger->total_process++;
 
    /* Make compiler happy */
    memset (&log, 0, sizeof (log));
-   if (parse_request (&log, line, cpy_line) != 0) {
+   if (parse_request (&log, line, cpy_line, &status_code, &req) != 0) {
       free (cpy_line);
       logger->total_invalid++;
       return 0;
@@ -499,7 +553,8 @@ process_log (struct logger *logger, char *line)
 
    process_unique_data (log.host, log.date, log.agent, log.status,
                         log.referrer);
-   free (status_code);
+   if (http_status_code_flag)
+      free (status_code);
    if (verify_static_content (log.request)) {
       if (strstr (cpy_line, "\" 404 "))
          process_generic_data (ht_not_found_requests, log.request);
@@ -511,7 +566,7 @@ process_log (struct logger *logger, char *line)
    }
    process_generic_data (ht_referrers, log.referrer);
    if (bandwidth_flag)
-      process_request_bw (ht_file_bw, log.request, log.resp_size);
+      process_request_bw (log.host, log.date, log.request, log.resp_size);
    free (cpy_line);
    free (req);
    return 0;
@@ -540,47 +595,42 @@ parse_log (struct logger *logger, char *filename, char *tail)
    return 0;
 }
 
+static void
+unique_visitors_iter (gpointer k, gpointer v, struct struct_holder **t_holder)
+{
+   t_holder[iter_ctr] = malloc (sizeof (struct struct_holder));
+   t_holder[iter_ctr]->data = (gchar *) k;
+   t_holder[iter_ctr++]->hits = GPOINTER_TO_INT (v);
+}
+
 void
 generate_unique_visitors (struct struct_display **s_display,
                           struct logger *logger)
 {
-   int lo, r = 0, w = 0;
+   int i, r;
+   int s_size = g_hash_table_size (ht_unique_vis);
    struct struct_holder **t_holder;
 
-   GHashTableIter iter;
-   gpointer k = NULL;
-   gpointer v = NULL;
+   t_holder = malloc (sizeof (struct struct_holder *) * s_size);
+   g_hash_table_foreach (ht_unique_vis, (GHFunc) unique_visitors_iter,
+                         t_holder);
 
-   int ct = 0;
-   t_holder =
-      (struct struct_holder **) malloc (sizeof (struct struct_holder *) *
-                                        g_hash_table_size (ht_unique_vis));
-
-   g_hash_table_iter_init (&iter, ht_unique_vis);
-   while (g_hash_table_iter_next (&iter, &k, &v)) {
-      t_holder[w] =
-         (struct struct_holder *) malloc (sizeof (struct struct_holder));
-      t_holder[w]->data = (gchar *) k;
-      t_holder[w++]->hits = GPOINTER_TO_INT (v);
-      ct++;
-   }
-
-   qsort (t_holder, ct, sizeof (struct struct_holder *), struct_cmp_desc);
+   qsort (t_holder, s_size, sizeof (struct struct_holder *), struct_cmp_desc);
 
    /* fixed value in here, perhaps it could be dynamic depending on the module */
-   for (lo = 0; lo < 10; lo++) {
+   for (i = 0, r = 0; i < 10; i++) {
       s_display[logger->alloc_counter]->hits = 0;
       s_display[logger->alloc_counter]->module = 1;
-      if (lo == 0)
+      if (i == 0)
          s_display[logger->alloc_counter++]->data =
             alloc_string (" 1 - Unique visitors per day - Including spiders");
-      else if (lo == 1)
+      else if (i == 1)
          s_display[logger->alloc_counter++]->data =
             alloc_string
             (" HTTP requests having the same IP, same date and same agent will be considered a unique visit");
-      else if (lo == 2 || lo == 9)
+      else if (i == 2 || i == 9)
          s_display[logger->alloc_counter++]->data = alloc_string ("");
-      else if (r < ct) {
+      else if (r < s_size) {
          s_display[logger->alloc_counter]->hits = t_holder[r]->hits;
          s_display[logger->alloc_counter++]->data =
             alloc_string (t_holder[r]->data);
@@ -589,11 +639,21 @@ generate_unique_visitors (struct struct_display **s_display,
          s_display[logger->alloc_counter++]->data = alloc_string ("");
    }
 
-   int f;
-   for (f = 0; f < ct; f++)
-      free (t_holder[f]);
+   for (i = 0; i < s_size; i++)
+      free (t_holder[i]);
    free (t_holder);
-   logger->counter = 0;
+
+   iter_ctr = 0;
+}
+
+static void
+struct_data_iter (gpointer k, gpointer v, struct struct_holder **s_holder)
+{
+   if (iter_module == BROWSERS || iter_module == OS)
+      if (strchr ((gchar *) k, '|') != NULL)
+         return;
+   s_holder[iter_ctr]->data = (gchar *) k;
+   s_holder[iter_ctr++]->hits = GPOINTER_TO_INT (v);
 }
 
 void
@@ -603,25 +663,12 @@ generate_struct_data (GHashTable * hash_table,
                       struct logger *logger, int module)
 {
    int row, col;
-
+   int i = 0, r;
    getmaxyx (stdscr, row, col);
 
-   int i = 0;
-   GHashTableIter iter;
-   gpointer k = NULL;
-   gpointer v = NULL;
-
-   g_hash_table_iter_init (&iter, hash_table);
-   while (g_hash_table_iter_next (&iter, &k, &v)) {
-      if (module == BROWSERS || module == OS)
-         if (strchr ((gchar *) k, '|') != NULL)
-            continue;
-      s_holder[i]->data = (gchar *) k;
-      s_holder[i++]->hits = GPOINTER_TO_INT (v);
-      logger->counter++;
-   }
-
-   qsort (s_holder, logger->counter, sizeof (struct struct_holder *),
+   iter_module = module;
+   g_hash_table_foreach (hash_table, (GHFunc) struct_data_iter, s_holder);
+   qsort (s_holder, iter_ctr, sizeof (struct struct_holder *),
           struct_cmp_by_hits);
 
    /* headers & sub-headers */
@@ -671,21 +718,16 @@ generate_struct_data (GHashTable * hash_table,
        break;
    }
 
-   /* r : pos on y */
-   char *stripped_str = NULL;
-   int lo, r = 0;
-   guint f;
-
-   for (lo = 0; lo < 10; lo++) {
+   for (i = 0, r = 0; i < 10; i++) {
       s_display[logger->alloc_counter]->hits = 0;
       s_display[logger->alloc_counter]->module = module;
-      if (lo == 0)
+      if (i == 0)
          s_display[logger->alloc_counter++]->data = alloc_string (head);
-      else if (lo == 1)
+      else if (i == 1)
          s_display[logger->alloc_counter++]->data = alloc_string (desc);
-      else if (lo == 2 || lo == 9)
+      else if (i == 2 || i == 9)
          s_display[logger->alloc_counter++]->data = alloc_string ("");
-      else if (r < logger->counter) {
+      else if (r < iter_ctr) {
          s_display[logger->alloc_counter]->data =
             alloc_string (s_holder[r]->data);
          s_display[logger->alloc_counter++]->hits = s_holder[r]->hits;
@@ -694,8 +736,8 @@ generate_struct_data (GHashTable * hash_table,
          s_display[logger->alloc_counter++]->data = alloc_string ("");
    }
 
-   for (f = 0; f < g_hash_table_size (hash_table); f++)
-      free (s_holder[f]);
+   for (i = 0; i < g_hash_table_size (hash_table); i++)
+      free (s_holder[i]);
    free (s_holder);
-   logger->counter = 0;
+   iter_ctr = 0;
 }
