@@ -30,25 +30,27 @@
 #define _XOPEN_SOURCE 700
 #define STDIN_FILENO  0
 
-#include <string.h>
 #include <curses.h>
-#include <time.h>
-#include <menu.h>
-#include <glib.h>
-#include <stdlib.h>
 #include <GeoIP.h>
-#include <unistd.h>
-#include <sys/stat.h>
+#include <glib.h>
 #include <math.h>
+#include <menu.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <time.h>
+#include <unistd.h>
 
 #include "parser.h"
 #include "alloc.h"
 #include "commons.h"
 #include "util.h"
 #include "ui.h"
+#include "error.h"
+#include "goaccess.h"
 
-static MENU *my_menu = NULL;
 static ITEM **items = NULL;
+static MENU *my_menu = NULL;
 
 /* creation - ncurses' window handling */
 WINDOW *
@@ -59,6 +61,17 @@ create_win (WINDOW * main_win)
    return (newwin (y - 12, x - 40, 8, 20));
 }
 
+void
+set_input_opts (void)
+{
+   noecho ();
+   halfdelay (10);
+   nonl ();
+   intrflush (stdscr, FALSE);
+   keypad (stdscr, TRUE);
+   curs_set (0);
+}
+
 /* delete ncurses window handling */
 void
 close_win (WINDOW * w)
@@ -172,14 +185,26 @@ display_general (WINDOW * header_win, struct logger *logger, char *ifile)
    wattroff (header_win, COLOR_PAIR (COL_WHITE));
 }
 
-void
+static char *
+ht_bw_str (GHashTable * ht, const char *key)
+{
+   gpointer value_ptr;
+
+   value_ptr = g_hash_table_lookup (ht, key);
+   if (value_ptr != NULL)
+      return filesize_str (*(long long *) value_ptr);
+   else
+      return alloc_string ("-");
+}
+
+static void
 create_graphs (WINDOW * main_win, struct struct_display **s_display,
                struct logger *logger, int i, int module, int max)
 {
-   struct tm tm;
-   int x, y, xx, r, col, row;
-   float l_bar, scr_cal, orig_cal;
    char buf[12] = "";           /* date */
+   float l_bar, scr_cal, orig_cal;
+   int x, y, xx, r, col, row;
+   struct tm tm;
 
    GHashTable *hash_table = NULL;
 
@@ -200,16 +225,17 @@ create_graphs (WINDOW * main_win, struct struct_display **s_display,
        break;
    }
 
-   int inc_pos_x = 0, magn = 0;
+   int padding_rx = 0, magn = 0;
+   int padding_gx = 0;          /* only graph padding */
 
    magn = floor (log10 (logger->total_process));
    if (magn >= 7)
-      inc_pos_x = magn % 5;
+      padding_rx = magn % 5;
    else
-      inc_pos_x = 0;
+      padding_rx = 0;
 
-   orig_cal = (float) (s_display[i]->hits * 100);
    l_bar = (float) (s_display[i]->hits * 100);
+   orig_cal = (float) (s_display[i]->hits * 100);
 
    /* hosts - status codes */
    if (module != 8 && module != 9)
@@ -222,19 +248,27 @@ create_graphs (WINDOW * main_win, struct struct_display **s_display,
    if (s_display[i]->module == 1) {
       strptime (s_display[i]->data, "%Y%m%d", &tm);
       strftime (buf, sizeof (buf), "%d/%b/%Y", &tm);
-      mvwprintw (main_win, y, 18 + inc_pos_x, "%s", buf);
+      mvwprintw (main_win, y, 18 + padding_rx, "%s", buf);
+      /* bandwidth */
+      if (bandwidth_flag) {
+         char *bw = ht_bw_str (ht_date_bw, buf);
+         wattron (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
+         mvwprintw (main_win, y, 31 + padding_rx, "%9s", bw);
+         wattroff (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
+         free (bw);
+      }
    } else if (s_display[i]->module == 9)
       /* HTTP status codes */
-      mvwprintw (main_win, y, 18 + inc_pos_x, "%s\t%s",
+      mvwprintw (main_win, y, 18 + padding_rx, "%s\t%s",
                  s_display[i]->data, verify_status_code (s_display[i]->data));
    else
-      mvwprintw (main_win, y, 18 + inc_pos_x, "%s", s_display[i]->data);
+      mvwprintw (main_win, y, 18 + padding_rx, "%s", s_display[i]->data);
    mvwprintw (main_win, y, 2, "%d", s_display[i]->hits);
    if (s_display[i]->hits == max)
       wattron (main_win, COLOR_PAIR (COL_YELLOW));
    else
       wattron (main_win, COLOR_PAIR (COL_RED));
-   mvwprintw (main_win, y, 10 + inc_pos_x, "%4.2f%%", orig_cal);
+   mvwprintw (main_win, y, 10 + padding_rx, "%4.2f%%", orig_cal);
    if (s_display[i]->hits == max)
       wattroff (main_win, COLOR_PAIR (COL_YELLOW));
    else
@@ -242,17 +276,21 @@ create_graphs (WINDOW * main_win, struct struct_display **s_display,
    if (s_display[i]->module == 9)
       return;
 
-   scr_cal = (float) ((col - 38) - inc_pos_x);
+   if (bandwidth_flag && s_display[i]->module == 1)
+      padding_gx = 44;
+   else
+      padding_gx = 36;
+   scr_cal = (float) ((col - padding_gx) - padding_rx);
    scr_cal = (float) scr_cal / 100;
    l_bar = l_bar * scr_cal;
 
    wattron (main_win, COLOR_PAIR (COL_GREEN));
-   for (r = 0, xx = 35 + inc_pos_x; r < (int) l_bar; r++, xx++)
+   for (r = 0, xx = padding_gx - 2 + padding_rx; r < (int) l_bar; r++, xx++)
       mvwprintw (main_win, y, xx, "|");
    wattroff (main_win, COLOR_PAIR (COL_GREEN));
 }
 
-int
+static int
 get_max_value (struct struct_display **s_display, struct logger *logger,
                int module)
 {
@@ -271,7 +309,7 @@ data_by_total_hits (WINDOW * main_win, int pos_y, struct logger *logger,
                     struct struct_display **s_display, int i)
 {
    float h = 0, t = 0;
-   int inc_pos_x = 0, magn = 0, col, row;
+   int padding_rx = 0, magn = 0, col, row;
    getmaxyx (main_win, row, col);
 
    if (s_display[i]->hits == 0)
@@ -282,53 +320,44 @@ data_by_total_hits (WINDOW * main_win, int pos_y, struct logger *logger,
 
    magn = floor (log10 (logger->total_process));
    if (magn >= 7)
-      inc_pos_x = magn % 5;
+      padding_rx = magn % 5;
    else
-      inc_pos_x = 0;
+      padding_rx = 0;
 
-   gpointer value_ptr;
-   long long *ptr_value;
+   char *c_data = s_display[i]->data;   /* current m_data */
+   int c_mod = s_display[i]->module;    /* current module */
 
-   if (bandwidth_flag
-       && (s_display[i]->module == REQUESTS
-           || s_display[i]->module == REQUESTS_STATIC)) {
-      value_ptr = g_hash_table_lookup (ht_file_bw, s_display[i]->data);
-      if (value_ptr != NULL) {
-         ptr_value = (long long *) value_ptr;
-         char *bw = filesize_str (*ptr_value);
-         wattron (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
-         mvwprintw (main_win, pos_y, 18 + inc_pos_x, "%9s", bw);
-         wattroff (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
-         if (strlen (s_display[i]->data) > ((size_t) (col - 32))) {
-            char *str = (char *) malloc (col - 32 + 1);
-            if (str == NULL)
-               error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
-                              "Unable to allocate memory");
-            strncpy (str, s_display[i]->data, col - 32);
-            (str)[col - 32] = 0;
-            mvwprintw (main_win, pos_y, 29 + inc_pos_x, "%s", str);
-            free (str);
-         } else
-            mvwprintw (main_win, pos_y, 29 + inc_pos_x, "%s",
-                       s_display[i]->data);
-         free (bw);
-      }
-   } else if (strlen (s_display[i]->data) > ((size_t) (col - 22))) {
+   if (bandwidth_flag && (c_mod == REQUESTS || c_mod == REQUESTS_STATIC)) {
+      char *bw = ht_bw_str (ht_file_bw, c_data);
+      wattron (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
+      mvwprintw (main_win, pos_y, 18 + padding_rx, "%9s", bw);
+      wattroff (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
+      if (strlen (c_data) > ((size_t) (col - 32))) {
+         char *str = (char *) malloc (col - 32 + 1);
+         if (str == NULL)
+            error_handler (__PRETTY_FUNCTION__, __FILE__,
+                           __LINE__, "Unable to allocate memory");
+         strncpy (str, c_data, col - 32);
+         (str)[col - 32] = 0;
+         mvwprintw (main_win, pos_y, 29 + padding_rx, "%s", str);
+         free (str);
+      } else
+         mvwprintw (main_win, pos_y, 29 + padding_rx, "%s", c_data);
+      free (bw);
+   } else if (strlen (c_data) > ((size_t) (col - 22))) {
       char *str = (char *) malloc (col - 22 + 1);
       if (str == NULL)
-         error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
-                        "Unable to allocate memory");
-      strncpy (str, s_display[i]->data, col - 22);
+         error_handler (__PRETTY_FUNCTION__, __FILE__,
+                        __LINE__, "Unable to allocate memory");
+      strncpy (str, c_data, col - 22);
       (str)[col - 22] = 0;
-      mvwprintw (main_win, pos_y, 18 + inc_pos_x, "%s", str);
+      mvwprintw (main_win, pos_y, 18 + padding_rx, "%s", str);
       free (str);
    } else
-      mvwprintw (main_win, pos_y, 18 + inc_pos_x, "%s", s_display[i]->data);
-
-   mvwprintw (main_win, pos_y, 2, "%d", s_display[i]->hits);
+      mvwprintw (main_win, pos_y, 18 + padding_rx, "%s", c_data);
 
    wattron (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
-   mvwprintw (main_win, pos_y, 10 + inc_pos_x, "%4.2f%%", t);
+   mvwprintw (main_win, pos_y, 10 + padding_rx, "%4.2f%%", t);
    wattroff (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
 }
 
@@ -340,8 +369,8 @@ display_content (WINDOW * main_win, struct struct_display **s_display,
 {
    int i, x, y, max = 0, until = 0, start = 0, pos_y = 0;
 
-   getmaxyx (stdscr, term_h, term_w);
    getmaxyx (main_win, y, x);
+   getmaxyx (stdscr, term_h, term_w);
 
    if (term_h < MIN_HEIGHT || term_w < MIN_WIDTH)
       error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
@@ -350,7 +379,7 @@ display_content (WINDOW * main_win, struct struct_display **s_display,
    if (logger->alloc_counter > real_size_y)
       until = real_size_y + scrolling.init_scrl_main_win;
    else
-      logger->alloc_counter;
+      until = logger->alloc_counter;
 
    start = scrolling.init_scrl_main_win;
 
@@ -408,8 +437,8 @@ do_scrolling (WINDOW * main_win, struct struct_display **s_display,
               struct logger *logger, struct scrolling *scrolling, int cmd)
 {
    int cur_y, cur_x, y, x, max = 0;
-   getyx (main_win, cur_y, cur_x);      /* cursor */
    getmaxyx (main_win, y, x);
+   getyx (main_win, cur_y, cur_x);      /* cursor */
 
    int i = real_size_y + scrolling->init_scrl_main_win;
    int j = scrolling->init_scrl_main_win - 1;
@@ -558,10 +587,10 @@ load_help_popup_content (WINDOW * inner_win, int where,
 void
 load_help_popup (WINDOW * help_win)
 {
-   WINDOW *inner_win;
    int y, x, c, quit = 1;
    size_t sz;
    struct scrolling scrolling;
+   WINDOW *inner_win;
 
    getmaxyx (help_win, y, x);
    draw_header (help_win,
@@ -599,41 +628,181 @@ load_help_popup (WINDOW * help_win)
    render_screens ();
 }
 
-void
+static void
+scrl_agent_win (WINDOW * inner_win, int where, struct scrolling *scrolling,
+                struct struct_agents *s_agents, size_t alloc)
+{
+   int y, x;
+   getmaxyx (inner_win, y, x);
+
+   switch (where) {
+       /* scroll down */
+    case 1:
+       if ((scrolling->scrl_agen_win > alloc - 1)
+           || (s_agents[scrolling->scrl_agen_win].agents == NULL))
+          return;
+       scrollok (inner_win, TRUE);
+       wscrl (inner_win, 1);
+       scrollok (inner_win, FALSE);
+       wmove (inner_win, y - 1, 2);
+       /* minus help_win offset - 0 */
+       mvwaddstr (inner_win, y - 1, 1,
+                  s_agents[scrolling->scrl_agen_win].agents);
+       scrolling->scrl_agen_win++;
+       break;
+       /* scroll up */
+    case 0:
+       if (scrolling->scrl_agen_win - y < 1)
+          return;
+       scrollok (inner_win, TRUE);
+       wscrl (inner_win, -1);
+       scrollok (inner_win, FALSE);
+       wmove (inner_win, 0, 2);
+       /* minus help_win offset - y */
+       mvwaddstr (inner_win, 0, 1,
+                  s_agents[(scrolling->scrl_agen_win - y) - 1].agents);
+       scrolling->scrl_agen_win--;
+       break;
+   }
+   wrefresh (inner_win);
+}
+
+/* split agent str if length > max or if '|' is found */
+static void
+split_agent_str (char *ptr_value, struct struct_agents *s_agents, size_t max)
+{
+   char *holder;
+   char *p = ptr_value;
+   int i = 0, offset = 0;
+
+   s_agents[i].agents = malloc (max + 1);
+   if (s_agents[i].agents == NULL)
+      error_handler (__PRETTY_FUNCTION__, __FILE__,
+                     __LINE__, "Unable to allocate memory.");
+   holder = s_agents[i].agents;
+
+   while (*p != '\0') {
+      if (offset < max && *p != '|') {
+         *(holder++) = *(p++);
+         offset++;
+         continue;
+      } else if (*p == '|')
+         (void) *p++;
+      offset = 0;
+      *holder = '\0';
+      s_agents[++i].agents = malloc (max + 1);
+      holder = s_agents[i].agents;
+   }
+   *holder = '\0';
+}
+
+static void
 load_reverse_dns_popup (WINDOW * ip_detail_win, char *addr)
 {
+   char *my_addr = NULL;
+   const char *location = NULL;
    int y, x, c, quit = 1;
-   char *my_addr = reverse_ip (addr);
-   const char *location;
+   struct scrolling scrolling;
+   struct struct_agents *s_agents;
+   WINDOW *inner_win;
 
+   /* make compiler happy */
+   memset (&s_agents, 0, sizeof (s_agents));
+
+   my_addr = reverse_ip (addr); /* reverse dns */
    getmaxyx (ip_detail_win, y, x);
-   draw_header (ip_detail_win, "  Reverse DNS lookup - q:quit", 0, 1, x - 1,
+   draw_header (ip_detail_win, " Reverse DNS lookup - q:quit", 1, 1, x - 2,
                 2);
    wborder (ip_detail_win, '|', '|', '-', '-', '+', '+', '+', '+');
    mvwprintw (ip_detail_win, 3, 2, "Reverse DNS for address: %s", addr);
    mvwprintw (ip_detail_win, 4, 2, "%s", my_addr);
+   free (my_addr);
 
-   /* geolocation data */
+   /* Geolocation data */
    GeoIP *gi;
    gi = GeoIP_new (GEOIP_STANDARD);
-   location = GeoIP_country_name_by_name (gi, addr);
-   GeoIP_delete (gi);
+   if (gi != NULL)
+      location = GeoIP_country_name_by_name (gi, addr);
    if (location == NULL)
       location = "Not found";
    mvwprintw (ip_detail_win, 5, 2, "Country: %s", location);
-   free (my_addr);
+   if (gi != NULL)
+      GeoIP_delete (gi);
 
+   draw_header (ip_detail_win, " [List of User-Agents]", 1, 7, x - 2, 2);
+   mvwprintw (ip_detail_win, 9, 2, "N/A");
+
+   char *ptr_value;
+   gpointer value_ptr = NULL;
+
+   int i, m, delims = 0;
+   size_t alloc = 0, width_max = 0, inner_y, inner_x;
+
+   /* agents' inner win */
+   inner_win = newwin (y - 10, x - 2, 19, 22);
+   if (inner_win == NULL)
+      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                     "Unable to allocate memory for new window.");
+   getmaxyx (inner_win, inner_y, inner_x);
+
+   /* List of User-Agents by host */
+   if (!host_agents_list_flag)
+      goto noagentlist;
+
+   value_ptr = g_hash_table_lookup (ht_hosts_agents, addr);
+   if (value_ptr != NULL) {
+      ptr_value = (char *) value_ptr;
+
+      delims = count_occurrences (ptr_value, '|');
+      width_max = inner_x - 4;
+      /* round-up + padding */
+      alloc = ((strlen (ptr_value) + width_max - 1) / width_max) + delims + 1;
+      s_agents = malloc (alloc * sizeof (s_agents));
+      if (s_agents == NULL)
+         error_handler (__PRETTY_FUNCTION__, __FILE__,
+                        __LINE__, "Unable to allocate memory.");
+      memset (s_agents, 0, alloc * sizeof (s_agents));
+
+      split_agent_str (ptr_value, s_agents, width_max);
+      for (i = 0, m = 0; (i < inner_y) && (s_agents[i].agents != NULL);
+           i++, m++)
+         mvwaddstr (inner_win, m, 1, s_agents[i].agents);
+   }
+
+ noagentlist:;
+
+   scrolling.scrl_agen_win = inner_y;
+   wmove (inner_win, inner_y, 0);
    wrefresh (ip_detail_win);
+   wrefresh (inner_win);
+
    /* ###TODO: resize child windows. */
    /* for now we can close them up */
    while (quit) {
       c = wgetch (stdscr);
       switch (c) {
+       case KEY_DOWN:
+          if ((value_ptr == NULL) || (alloc == 0))
+             break;
+          (void) scrl_agent_win (inner_win, 1, &scrolling, s_agents, alloc);
+          break;
+       case KEY_UP:
+          if ((value_ptr == NULL) || (alloc == 0))
+             break;
+          (void) scrl_agent_win (inner_win, 0, &scrolling, s_agents, alloc);
+          break;
        case KEY_RESIZE:
        case 'q':
           quit = 0;
           break;
       }
+      wrefresh (ip_detail_win);
+   }
+
+   if (host_agents_list_flag) {
+      for (i = 0; (value_ptr != NULL && s_agents[i].agents != NULL); i++)
+         free (s_agents[i].agents);
+      free (s_agents);
    }
    render_screens ();
    return;
@@ -653,11 +822,11 @@ scheme_chosen (char *name)
 void
 load_schemes_win (WINDOW * schemes_win)
 {
-   int y, x, c, quit = 1, n_choices, i;
    char *choices[] = { "Monochrome/Default", "Green/Original" };
-   MENU *menu;
-   ITEM **my_items;
+   int y, x, c, quit = 1, n_choices, i;
    ITEM *cur_item;
+   ITEM **my_items;
+   MENU *menu;
 
    /* Create items */
    n_choices = ARRAY_SIZE (choices);
@@ -732,7 +901,7 @@ get_browser_type (char *line)
    char *p;
    char token[128];
 
-   if (p = strchr (line, '|')) {
+   if ((p = strchr (line, '|')) != NULL) {
       if (sscanf (line, "%[^|]", token) == 1)
          return line + strlen (token) + 1;
       else {
@@ -749,72 +918,203 @@ convert_hits_to_string (int nhits)
    char *hits = NULL;
    hits = (char *) malloc (sizeof (char) * 11);
    if (hits == NULL)
-      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
-                     "Unable to allocate memory");
+      error_handler (__PRETTY_FUNCTION__,
+                     __FILE__, __LINE__, "Unable to allocate memory");
    sprintf (hits, "%3i", nhits);
    return hits;
 }
 
-static ITEM **
-get_menu_items (struct struct_holder **s_holder, struct logger *logger,
-                int choices, int sort)
+static void
+process_monthly (struct struct_holder **s_holder, int n_months)
 {
-   int i;
-   char *hits = NULL;
-   char buf[12] = "";
-   char *buffer_date = NULL, *b_version = NULL, *o_version = NULL;
-   char *p = NULL, *b;
-   struct tm tm;
-   ITEM **items;
-
-   /* requests module */
+   /* remove old keys/vals if exist */
+   g_hash_table_destroy (ht_monthly);
+   ht_monthly =
+      g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free,
+                             g_free);
    gpointer value_ptr;
-   long long *ptr_value;
-   char *bw, *w_bw, *status_code, *token, *status_str;
+   int add_value, i;
+   int *ptr_value = 0;
+
+   for (i = 0; i < n_months; i++) {
+      char *month = clean_month (s_holder[i]->data);
+      value_ptr = g_hash_table_lookup (ht_monthly, month);
+      if (value_ptr != NULL)
+         add_value = (int) *ptr_value + s_holder[i]->hits;
+      else
+         add_value = 0 + s_holder[i]->hits;
+
+      ptr_value = g_malloc (sizeof (int));
+      *ptr_value = add_value;
+
+      g_hash_table_replace (ht_monthly, g_strdup (month), ptr_value);
+      free (month);
+   }
+}
+
+static char *
+subgraphs (struct struct_holder **s_holder, int curr, size_t x, int max)
+{
+   char *buf;
+   int len_bar = 0;
+   size_t width = (x / 2) - 13;
+
+   if ((max == 0) || (x == 0))
+      return alloc_string ("");
+
+   len_bar = ((curr * width) / max);
+
+   buf = malloc (len_bar + 1);
+   if (buf == NULL)
+      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                     "Unable to allocate memory");
+   memset (buf, '|', len_bar);
+   buf[len_bar] = '\0';
+
+   return buf;
+}
+
+static char *
+set_month_data_unique_visitors (char *data)
+{
+   char buf[DATELEN] = "";
+   char *buffer_month;
+   struct tm tm;
 
    memset (&tm, 0, sizeof (tm));
 
-   /* sort struct prior to display */
-   if (sort)
-      qsort (s_holder, logger->counter, sizeof (struct struct_holder *),
-             struct_cmp_by_hits);
-   else
-      qsort (s_holder, logger->counter, sizeof (struct struct_holder *),
-             struct_cmp_desc);
+   if ((data == NULL))
+      return alloc_string ("-");
 
+   buffer_month = (char *) malloc (sizeof (char) * (DATELEN));
+   if (buffer_month == NULL)
+      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                     "Unable to allocate memory");
+
+   strptime (data, "%Y%m", &tm);
+   if (strftime (buf, sizeof (buf), "%b/%Y", &tm) <= 0)
+      *buf = 0;
+   strftime (buf, sizeof (buf), "%b/%Y", &tm);
+   sprintf (buffer_month, "%s", buf);
+
+   return buffer_month;
+}
+
+static char *
+set_nobw_data_unique_visitors (char *buf, char *graph)
+{
+   char *data;
+   int len;
+
+   if ((buf == NULL) || (graph == NULL))
+      return alloc_string ("-");
+
+   len = snprintf (NULL, 0, "|`- %s %s", buf, graph);
+   data = malloc (len + 2);
+   if (data == NULL)
+      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                     "Unable to allocate memory");
+   sprintf (data, "|`- %s %s", buf, graph);
+
+   return data;
+}
+
+static char *
+set_bw_data_unique_visitors (char *buf, char *graph)
+{
+   char *bw, *w_bw;
+   gpointer value_ptr;
+   int len;
+   long long *ptr_value;
+
+   value_ptr = g_hash_table_lookup (ht_date_bw, buf);
+   if (value_ptr == NULL)
+      return alloc_string ("-");
+
+   ptr_value = (long long *) value_ptr;
+   bw = filesize_str (*ptr_value);
+   len = snprintf (NULL, 0, "|`- %s %9s %s", buf, bw, graph);
+   w_bw = malloc (len + 3);
+   if (w_bw == NULL)
+      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                     "Unable to allocate memory");
+   sprintf (w_bw, "|`- %s %9s %s", buf, bw, graph);
+   free (bw);
+
+   return w_bw;
+}
+
+static ITEM **
+get_menu_items (struct struct_holder **s_holder, struct logger *logger,
+                int choices, int sort, WINDOW * my_menu_win, int max)
+{
+   char buf[DATELEN] = "";
+   char *b_version = NULL, *o_version = NULL;
+   char *hits = NULL, *graph = NULL;
+   ITEM **items;
+   struct tm tm;
+
+   char *bw, *w_bw, *status_code, *token, *status_str;
+   gpointer value_ptr;
+   int i;
+   long long *ptr_value;
+   size_t y, x;
+
+   memset (&tm, 0, sizeof (tm));
+
+   getmaxyx (my_menu_win, y, x);
+   /* sort struct prior to display */
    if (logger->current_module == BROWSERS || logger->current_module == OS)
-      qsort (s_holder, logger->counter, sizeof (struct struct_holder *),
-             struct_cmp_asc);
+      qsort (s_holder, choices, sizeof *s_holder, struct_cmp_asc);
+   else if (!sort && logger->current_module == UNIQUE_VISITORS)
+      qsort (s_holder, choices, sizeof *s_holder, struct_cmp_by_hits);
+   else if (logger->current_module == UNIQUE_VISITORS)
+      qsort (s_holder, choices, sizeof *s_holder, struct_cmp_desc);
 
    items = (ITEM **) malloc (sizeof (ITEM *) * (choices + 1));
 
+   int len;                     /* str len */
+   char *data = NULL;           /* s_holder[i]->data */
    for (i = 0; i < choices; ++i) {
       switch (logger->current_module) {
        case UNIQUE_VISITORS:
           hits = convert_hits_to_string (s_holder[i]->hits);
-          buffer_date = (char *) malloc (sizeof (char) * 13);
-          if (buffer_date == NULL)
-             error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
-                            "Unable to allocate memory");
-          strptime (s_holder[i]->data, "%Y%m%d", &tm);
-          strftime (buf, sizeof (buf), "%d/%b/%Y", &tm);
-          sprintf (buffer_date, "%s", buf);
-          items[i] = new_item (hits, buffer_date);
+          if (strchr (s_holder[i]->data, '|') == NULL && bandwidth_flag) {
+             graph = subgraphs (s_holder, s_holder[i]->hits, x, max);
+             convert_date (buf, s_holder[i]->data, DATELEN);
+             data = set_bw_data_unique_visitors (buf, graph);
+             items[i] = new_item (hits, data);
+             free (graph);
+             break;
+          } else if (strchr (s_holder[i]->data, '|') == NULL) {
+             graph = subgraphs (s_holder, s_holder[i]->hits, x + 13, max);
+             convert_date (buf, s_holder[i]->data, DATELEN);
+             data = set_nobw_data_unique_visitors (buf, graph);
+             items[i] = new_item (hits, data);
+             free (graph);
+             break;
+          }
+          data = set_month_data_unique_visitors (s_holder[i]->data);
+          items[i] = new_item (hits, data);
           break;
        case REQUESTS:
        case REQUESTS_STATIC:
+       case HOSTS:
           if (!bandwidth_flag) {
              hits = convert_hits_to_string (s_holder[i]->hits);
              items[i] = new_item (hits, s_holder[i]->data);
              break;
           }
-          value_ptr = g_hash_table_lookup (ht_file_bw, s_holder[i]->data);
+          if (logger->current_module == HOSTS)
+             value_ptr = g_hash_table_lookup (ht_host_bw, s_holder[i]->data);
+          else
+             value_ptr = g_hash_table_lookup (ht_file_bw, s_holder[i]->data);
           if (value_ptr == NULL)
              break;
           ptr_value = (long long *) value_ptr;
           bw = filesize_str (*ptr_value);
-          w_bw = malloc (snprintf (NULL, 0, "%9s - %d", bw, s_holder[i]->hits)
-                         + 2);
+          len = snprintf (NULL, 0, "%9s - %d", bw, s_holder[i]->hits);
+          w_bw = malloc (len + 2);
           if (w_bw == NULL)
              error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                             "Unable to allocate memory");
@@ -828,10 +1128,9 @@ get_menu_items (struct struct_holder **s_holder, struct logger *logger,
              items[i] = new_item (hits, alloc_string (s_holder[i]->data));
              break;
           }
-          o_version =
-             malloc (snprintf
-                     (NULL, 0, "|`- %s",
-                      get_browser_type (s_holder[i]->data)) + 1);
+          data = get_browser_type (s_holder[i]->data);
+          len = snprintf (NULL, 0, "|`- %s", data);
+          o_version = malloc (len + 1);
           if (o_version == NULL)
              error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                             "Unable to allocate memory");
@@ -844,10 +1143,9 @@ get_menu_items (struct struct_holder **s_holder, struct logger *logger,
              items[i] = new_item (hits, alloc_string (s_holder[i]->data));
              break;
           }
-          b_version =
-             malloc (snprintf
-                     (NULL, 0, "|`- %s",
-                      get_browser_type (s_holder[i]->data)) + 1);
+          data = get_browser_type (s_holder[i]->data);
+          len = snprintf (NULL, 0, "|`- %s", data);
+          b_version = malloc (len + 1);
           if (b_version == NULL)
              error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                             "Unable to allocate memory");
@@ -862,9 +1160,8 @@ get_menu_items (struct struct_holder **s_holder, struct logger *logger,
              error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                             "Unable to allocate memory");
           if (sscanf (status_code, "%[^-]", token) == 1) {
-             status_str =
-                malloc (snprintf
-                        (NULL, 0, "%s %s", s_holder[i]->data, token) + 1);
+             len = snprintf (NULL, 0, "%s %s", s_holder[i]->data, token);
+             status_str = malloc (len + 1);
              if (status_str == NULL)
                 error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                                "Unable to allocate memory");
@@ -880,17 +1177,17 @@ get_menu_items (struct struct_holder **s_holder, struct logger *logger,
           break;
       }
    }
-   items[i] = (ITEM *) NULL;
 
+   items[i] = (ITEM *) NULL;
    return items;
 }
 
 static MENU *
 set_menu (WINDOW * my_menu_win, ITEM ** items, struct logger *logger)
 {
-   MENU *my_menu = NULL;
    int x = 0;
    int y = 0;
+   MENU *my_menu = NULL;
 
    getmaxyx (my_menu_win, y, x);
    my_menu = new_menu (items);
@@ -898,16 +1195,22 @@ set_menu (WINDOW * my_menu_win, ITEM ** items, struct logger *logger)
 
    /* set main window and sub window */
    set_menu_win (my_menu, my_menu_win);
+   if (logger->current_module == UNIQUE_VISITORS)
+      mvwprintw (my_menu_win, 3, x - 30, "[Months:%4d - Days:%4d]",
+                 g_hash_table_size (ht_monthly),
+                 g_hash_table_size (ht_unique_vis));
+   if (logger->current_module == HOSTS)
+      mvwprintw (my_menu_win, 3, x - 31, "[Different Hosts:%8d]",
+                 g_hash_table_size (ht_hosts));
    set_menu_sub (my_menu, derwin (my_menu_win, y - 6, x - 2, 4, 1));
    set_menu_format (my_menu, y - 6, 1);
 
    /* set menu mark */
    set_menu_mark (my_menu, " => ");
-   draw_header (my_menu_win,
-                "  Use cursor UP/DOWN - PGUP/PGDOWN to scroll. q:quit", 0, 1,
-                x, 2);
-   draw_header (my_menu_win, module_names[logger->current_module - 1], 0, 2,
-                x, 1);
+   char *desc = module_names[logger->current_module - 1];
+   char *head = " Use cursor UP/DOWN - PGUP/PGDOWN to scroll. q:quit";
+   draw_header (my_menu_win, head, 1, 1, x, 2);
+   draw_header (my_menu_win, desc, 0, 2, x, 1);
    wborder (my_menu_win, '|', '|', '-', '-', '+', '+', '+', '+');
    return my_menu;
 }
@@ -915,10 +1218,10 @@ set_menu (WINDOW * my_menu_win, ITEM ** items, struct logger *logger)
 static void
 load_popup_content (WINDOW * my_menu_win, int choices,
                     struct struct_holder **s_holder, struct logger *logger,
-                    int sort)
+                    int sort, int max)
 {
    wclrtoeol (my_menu_win);
-   items = get_menu_items (s_holder, logger, choices, sort);
+   items = get_menu_items (s_holder, logger, choices, sort, my_menu_win, max);
    my_menu = set_menu (my_menu_win, items, logger);
    post_menu (my_menu);
    wrefresh (my_menu_win);
@@ -927,9 +1230,9 @@ load_popup_content (WINDOW * my_menu_win, int choices,
 static void
 load_popup_free_items (ITEM ** items, struct logger *logger)
 {
-   int i;
-   char *name = NULL;
    char *description = NULL;
+   char *name = NULL;
+   int i;
 
    /* clean up stuff */
    i = 0;
@@ -956,34 +1259,60 @@ search_request (MENU * my_menu, const char *input)
    const char *haystack;
    ITEM *item_ptr = NULL;
 
-   if (input != NULL) {
-      int i = -1, j = -1, response = 0;
-      j = item_index (current_item (my_menu));
+   if (input == NULL)
+      return item_ptr;
 
-      for (i = j + 1; i < item_count (my_menu) && !response; i++) {
-         haystack = item_description (menu_items (my_menu)[i]);
-         if (haystack != NULL && input != NULL) {
-            if (strstr (haystack, input))
-               response = 1;
-         } else
-            response = 0;
-      }
-      if (response)
-         item_ptr = menu_items (my_menu)[i - 1];
+   int i = -1, j = -1, response = 0;
+   j = item_index (current_item (my_menu));
+
+   for (i = j + 1; i < item_count (my_menu) && !response; i++) {
+      haystack = item_description (menu_items (my_menu)[i]);
+      if (haystack != NULL && input != NULL) {
+         if (strstr (haystack, input))
+            response = 1;
+      } else
+         response = 0;
    }
+   if (response)
+      item_ptr = menu_items (my_menu)[i - 1];
+
    return item_ptr;
 }
 
+static void
+load_popup_iter (gpointer k, gpointer v, struct struct_holder **s_holder)
+{
+   s_holder[iter_ctr]->data = (gchar *) k;
+   s_holder[iter_ctr++]->hits = GPOINTER_TO_INT (v);
+}
+
+static void
+load_popup_visitors_iter (gpointer k, gpointer v,
+                          struct struct_holder **s_holder)
+{
+   s_holder[iter_ctr] = malloc (sizeof *s_holder[iter_ctr]);
+   /* we assume we dont have 99 days in a month, */
+   /* this way we can sort them out and put them on top */
+   /* there might be a better way to go though :) */
+   char *s = malloc (snprintf (NULL, 0, "%s|99", (gchar *) k) + 1);
+   if (s == NULL)
+      error_handler (__PRETTY_FUNCTION__,
+                     __FILE__, __LINE__, "Unable to allocate memory");
+   sprintf (s, "%s|99", (gchar *) k);
+   s_holder[iter_ctr]->data = s;
+   s_holder[iter_ctr++]->hits = *(int *) v;
+}
+
 void
 load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
             struct logger *logger)
 {
-   WINDOW *ip_detail_win;
    ITEM *query = NULL;
+   WINDOW *ip_detail_win;
 
    /*###TODO: perhaps let the user change the size of MAX_CHOICES */
-   int choices = MAX_CHOICES, c, x, y;
    char input[BUFFER] = "";
+   int choices = MAX_CHOICES, c, x, y;
 
    GHashTable *hash_table = NULL;
 
@@ -1025,18 +1354,13 @@ load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
    getmaxyx (my_menu_win, y, x);
    MALLOC_STRUCT (s_holder, g_hash_table_size (hash_table));
 
-   int i = 0, quit = 1;
-   char *p;
-   GHashTableIter iter;
-   gpointer k = NULL;
-   gpointer v = NULL;
+   int hash_table_size = g_hash_table_size (hash_table);
+   int i = 0, quit = 1, max = 0;
+   int new_menu_size = 0;
 
-   g_hash_table_iter_init (&iter, hash_table);
-   while (g_hash_table_iter_next (&iter, &k, &v)) {
-      s_holder[i]->data = (gchar *) k;
-      s_holder[i++]->hits = GPOINTER_TO_INT (v);
-      logger->counter++;
-   }
+   g_hash_table_foreach (hash_table, (GHFunc) load_popup_iter, s_holder);
+   /* set it 0 for the next g_hash_table_foreach() */
+   iter_ctr = 0;
 
    /* again, letting the user to set the max number 
     * might be a better way to go */
@@ -1047,7 +1371,36 @@ load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
    else
       choices = g_hash_table_size (hash_table);
 
-   load_popup_content (my_menu_win, choices, s_holder, logger, 1);
+   /* sort s_holder */
+   qsort (s_holder, hash_table_size, sizeof *s_holder, struct_cmp_by_hits);
+   if (logger->current_module == UNIQUE_VISITORS) {
+      for (i = 0; i < g_hash_table_size (ht_unique_vis); i++) {
+         if (s_holder[i]->hits > max)
+            max = s_holder[i]->hits;
+      }
+      if (hash_table_size == 0)
+         goto err;
+      /* add up monthly history totals */
+      process_monthly (s_holder, hash_table_size);
+      /* realloc the extra space needed for months */
+      new_menu_size = (hash_table_size + g_hash_table_size (ht_monthly));
+      struct struct_holder **s_tmp;
+      s_tmp = realloc (s_holder, new_menu_size * sizeof *s_holder);
+      if (s_tmp == NULL)
+         error_handler (__PRETTY_FUNCTION__, __FILE__,
+                        __LINE__, "Unable to re-allocate memory");
+      else
+         s_holder = s_tmp;
+
+      iter_ctr = hash_table_size;
+      g_hash_table_foreach (ht_monthly, (GHFunc) load_popup_visitors_iter,
+                            s_holder);
+      choices = new_menu_size;
+    err:;
+      iter_ctr = 0;
+   }
+
+   load_popup_content (my_menu_win, choices, s_holder, logger, 1, max);
 
    while (quit) {
       c = wgetch (stdscr);
@@ -1077,12 +1430,7 @@ load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
           wscanw (my_menu_win, "%s", input);
           wattroff (my_menu_win, COLOR_PAIR (COL_CYAN));
           cbreak ();
-          noecho ();
-          halfdelay (10);
-          nonl ();
-          intrflush (stdscr, FALSE);
-          curs_set (0);
-
+          set_input_opts ();    /* set curses input options */
           query = search_request (my_menu, input);
           if (query != NULL) {
              while (FALSE == item_visible (query))
@@ -1123,7 +1471,7 @@ load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
           free_menu (my_menu);
           my_menu = NULL;
           load_popup_free_items (items, logger);
-          load_popup_content (my_menu_win, choices, s_holder, logger, 0);
+          load_popup_content (my_menu_win, choices, s_holder, logger, 0, max);
           break;
        case 'S':
           if (logger->current_module != UNIQUE_VISITORS)
@@ -1132,9 +1480,11 @@ load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
           free_menu (my_menu);
           my_menu = NULL;
           load_popup_free_items (items, logger);
-          load_popup_content (my_menu_win, choices, s_holder, logger, 1);
+          load_popup_content (my_menu_win, choices, s_holder, logger, 1, max);
           break;
-       case 10:
+       case 0x0a:
+       case 0x0d:
+       case KEY_ENTER:
        case KEY_RIGHT:
           if (logger->current_module != 8)
              break;
@@ -1144,7 +1494,7 @@ load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
           if (cur == NULL)
              break;
 
-          ip_detail_win = newwin (y - 13, x - 5, 10, 21);
+          ip_detail_win = newwin (y - 3, x - 2, 10, 21);
           char addrs[32];
           sprintf (addrs, "%s", item_description (cur));
           load_reverse_dns_popup (ip_detail_win, addrs);
@@ -1163,11 +1513,14 @@ load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
       wrefresh (my_menu_win);
    }
 
-   int f;
-   for (f = 0; f < logger->counter; f++)
-      free (s_holder[f]);
+   int free_n = (new_menu_size != 0) ? new_menu_size : hash_table_size;
+   for (i = 0; i < free_n; i++) {
+      if (logger->current_module == UNIQUE_VISITORS
+          && strchr (s_holder[i]->data, '|') != NULL)
+         free (s_holder[i]->data);
+      free (s_holder[i]);
+   }
    free (s_holder);
-   logger->counter = 0;
 
    /* unpost and free all the memory taken up */
    unpost_menu (my_menu);
