@@ -22,26 +22,28 @@
 #define UI_H_INCLUDED
 
 WINDOW *create_win (WINDOW * main_win);
+
+struct struct_agents
+{
+   char *agents;
+};
+
 void close_win (WINDOW * w);
-void generate_time (void);
-void draw_header (WINDOW * win, char *header, int x, int y, int w, int color);
-void update_header (WINDOW * header_win, int current);
-void term_size (WINDOW * main_win);
-void display_general (WINDOW * header_win, struct logger *logger,
-                      char *ifile);
-void create_graphs (WINDOW * main_win, struct struct_display **s_display,
-                    struct logger *logger, int i, int module, int max);
-int get_max_value (struct struct_display **s_display, struct logger *logger,
-                   int module);
 void display_content (WINDOW * main_win, struct struct_display **s_display,
                       struct logger *logger, struct scrolling scrolling);
+void display_general (WINDOW * header_win, struct logger *logger,
+                      char *ifile);
 void do_scrolling (WINDOW * main_win, struct struct_display **s_display,
                    struct logger *logger, struct scrolling *scrolling,
                    int cmd);
+void draw_header (WINDOW * win, char *header, int x, int y, int w, int color);
+void generate_time (void);
 void load_help_popup (WINDOW * help_win);
-void load_schemes_win (WINDOW * schemes_win);
-void load_reverse_dns_popup (WINDOW * ip_detail_win, char *addr);
 void load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
                  struct logger *logger);
+void load_schemes_win (WINDOW * schemes_win);
+void set_input_opts (void);
+void term_size (WINDOW * main_win);
+void update_header (WINDOW * header_win, int current);
 
 #endif
